<?php

namespace Drupal\easy_google_analytics_counter;

use Drupal\Core\Database\Driver\mysql\Connection;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Path\AliasManagerInterface;
// --------- Use from google analytics library ----------.
use Google_Client;
use Google_Service_AnalyticsReporting;
use Google_Service_AnalyticsReporting_DateRange;
use Google_Service_AnalyticsReporting_Metric;
use Google_Service_AnalyticsReporting_ReportRequest;
use Google_Service_AnalyticsReporting_GetReportsRequest;
use Google_Service_AnalyticsReporting_Dimension;
use Google_Service_AnalyticsReporting_OrderBy;

/**
 * Class ConnectionService.
 */
class ConnectionService implements ConnectionServiceInterface {

  /**
   * Drupal\Core\Database\Driver\mysql\Connection definition.
   *
   * @var \Drupal\Core\Database\Driver\mysql\Connection
   */
  protected $database;

  /**
   * Drupal\Core\Config\ConfigFactoryInterface definition.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $config;

  /**
   * Drupal\Core\Entity\EntityTypeManager definition.
   *
   * @var \Drupal\Core\Entity\EntityTypeManager
   */
  protected $entityTypeManager;

  /**
   * Drupal\Core\File\FileSystemInterface definition.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * Drupal\Core\Path\AliasManagerInterface definition.
   *
   * @var \Drupal\Core\Path\AliasManagerInterface
   */
  protected $aliasManager;

  /**
   * Google_Service_AnalyticsReporting definition.
   *
   * @var \Google_Service_AnalyticsReporting
   */
  protected $analytics;

  /**
   * Constructs a new ConnectionService object.
   */
  public function __construct(Connection $database, ConfigFactoryInterface $config_factory, EntityTypeManagerInterface $entity_type_manager, FileSystemInterface $file_system, AliasManagerInterface $alias_manager) {
    $this->database = $database;
    $this->config = $config_factory->get('easy_google_analytics_counter.admin');
    $this->entityTypeManager = $entity_type_manager;
    $this->fileSystem = $file_system;
    $this->aliasManager = $alias_manager;
  }

  /**
   * Initialize the google analytics service.
   */
  protected function initializeAnalytics() {
    // Use the developers console and download your service account
    // credentials in JSON format. Place them in this directory or
    // change the key file location if necessary.
    // $key_file_location = __DIR__ . '/service-account-credentials.json';
    // Create and configure a new client object.
    $client = new Google_Client();
    $client->setApplicationName($this->config->get('application_name'));
    $client->setAuthConfig($this->getKeyLocation());
    $client->setScopes(['https://www.googleapis.com/auth/analytics.readonly']);
    $this->analytics = new Google_Service_AnalyticsReporting($client);
  }

  /**
   * {@inheritdoc}
   */
  public function request() {
    $this->initializeAnalytics();

    // Your view ID, defined on admin form.
    $view_id = $this->config->get('view_id');

    // Create the Dimensions object.
    $pagepath = new Google_Service_AnalyticsReporting_Dimension();
    $pagepath->setName('ga:pagePath');

    // Create the DateRange object.
    $days = $this->config->get('start_date');
    $start = strtotime("-$days day");
    $dateRange = new Google_Service_AnalyticsReporting_DateRange();
    $dateRange->setStartDate(date('Y-m-d', $start));
    $dateRange->setEndDate("today");

    // Create the Metrics object for Page Views.
    $pageViews = new Google_Service_AnalyticsReporting_Metric();
    $pageViews->setExpression('ga:pageviews');
    $pageViews->setAlias('pageviews');

    // Create the OrderBy object for Page Views descening.
    $orderBy = new Google_Service_AnalyticsReporting_OrderBy();
    $orderBy->setFieldName('ga:pageviews');
    $orderBy->setSortOrder('DESCENDING');

    // Create the ReportRequest object.
    $request = new Google_Service_AnalyticsReporting_ReportRequest();
    $request->setViewId($view_id);
    $request->setDateRanges($dateRange);
    $request->setMetrics([$pageViews]);
    $request->setDimensions([$pagepath]);
    $request->setOrderBys($orderBy);
    $request->setPageSize($this->config->get('number_items'));
    $body = new Google_Service_AnalyticsReporting_GetReportsRequest();
    $body->setReportRequests([$request]);

    $response = $this->analytics->reports->batchGet($body);
    $this->setData($response);
  }

  /**
   * Set pageviews data to database.
   *
   * @param array $reports
   *   The response array from GA.
   */
  protected function setData(array $reports) {
    for ($reportIndex = 0; $reportIndex < count($reports); $reportIndex++) {
      $report = $reports[$reportIndex];
      $header = $report->getColumnHeader();
      $dimensionHeaders = $header->getDimensions();
      $rows = $report->getData()->getRows();
      $matches = [];
      for ($rowIndex = 0; $rowIndex < count($rows); $rowIndex++) {
        $row = $rows[$rowIndex];
        $dimensions = $row->getDimensions();
        $metrics = $row->getMetrics();
        for ($i = 0; $i < count($dimensionHeaders) && $i < count($dimensions); $i++) {
          if ($dimensionHeaders[$i] === 'ga:pagePath') {
            $alias = $this->aliasManager->getPathByAlias($dimensions[$i]);
            if (preg_match('/node\/(\d+)/', $alias, $matches)) {
              $page_views = $metrics[0]->getValues();
              if (!empty($matches[1]) && isset($page_views[0])) {
                $this->database->update('node_field_data')
                  ->fields(['page_views' => $page_views[0]])
                  ->condition('nid', $matches[1])
                  ->execute();
              }
            }
          }
        }
      }
    }
  }

  /**
   * Return key file location.
   *
   * @return string
   *   The url of key file.
   */
  protected function getKeyLocation() {
    $url = '';
    $files = $this->config->get('service_account_credentials_json');
    if (!empty($files[0])) {
      if ($file = $this->entityTypeManager->getStorage('file')->load($files[0])) {
        $url = $this->fileSystem->realpath($file->uri->value);
      }
    }

    return $url;
  }

}
